#ifndef DESKTOP_SERVER_PROTOCOL_H
#define DESKTOP_SERVER_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-util.h"

struct wl_client;
struct wl_resource;

struct desktop_shell;
struct screensaver;
struct input_panel;

extern const struct wl_interface desktop_shell_interface;
extern const struct wl_interface screensaver_interface;
extern const struct wl_interface input_panel_interface;

/**
 * desktop_shell - create desktop widgets and helpers
 * @set_background: (none)
 * @set_panel: (none)
 * @set_lock_surface: (none)
 * @unlock: (none)
 * @set_busy_surface: (none)
 *
 * Traditional user interfaces can rely on this interface to define the
 * foundations of typical desktops. Currently it's possible to set up
 * background, panels and locking surfaces.
 */
struct desktop_shell_interface {
	/**
	 * set_background - (none)
	 * @output: (none)
	 * @surface: (none)
	 */
	void (*set_background)(struct wl_client *client,
			       struct wl_resource *resource,
			       struct wl_resource *output,
			       struct wl_resource *surface);
	/**
	 * set_panel - (none)
	 * @output: (none)
	 * @surface: (none)
	 */
	void (*set_panel)(struct wl_client *client,
			  struct wl_resource *resource,
			  struct wl_resource *output,
			  struct wl_resource *surface);
	/**
	 * set_lock_surface - (none)
	 * @surface: (none)
	 */
	void (*set_lock_surface)(struct wl_client *client,
				 struct wl_resource *resource,
				 struct wl_resource *surface);
	/**
	 * unlock - (none)
	 */
	void (*unlock)(struct wl_client *client,
		       struct wl_resource *resource);
	/**
	 * set_busy_surface - (none)
	 * @surface: (none)
	 */
	void (*set_busy_surface)(struct wl_client *client,
				 struct wl_resource *resource,
				 struct wl_resource *surface);
};

#define DESKTOP_SHELL_CONFIGURE	0
#define DESKTOP_SHELL_PREPARE_LOCK_SURFACE	1

static inline void
desktop_shell_send_configure(struct wl_resource *resource_, uint32_t edges, struct wl_resource *surface, int32_t width, int32_t height)
{
	wl_resource_post_event(resource_, DESKTOP_SHELL_CONFIGURE, edges, surface, width, height);
}

static inline void
desktop_shell_send_prepare_lock_surface(struct wl_resource *resource_)
{
	wl_resource_post_event(resource_, DESKTOP_SHELL_PREPARE_LOCK_SURFACE);
}

/**
 * screensaver - interface for implementing screensavers
 * @set_surface: set the surface type as a screensaver
 *
 * Only one client can bind this interface at a time.
 */
struct screensaver_interface {
	/**
	 * set_surface - set the surface type as a screensaver
	 * @surface: (none)
	 * @output: (none)
	 *
	 * A screensaver surface is normally hidden, and only visible
	 * after an idle timeout.
	 */
	void (*set_surface)(struct wl_client *client,
			    struct wl_resource *resource,
			    struct wl_resource *surface,
			    struct wl_resource *output);
};

/**
 * input_panel - interface for implementing keyboards
 * @set_surface: set the surface type as a keyboard
 *
 * Only one client can bind this interface at a time.
 */
struct input_panel_interface {
	/**
	 * set_surface - set the surface type as a keyboard
	 * @surface: (none)
	 * @output: (none)
	 *
	 * A keybaord surface is only shown, when a text model is active
	 */
	void (*set_surface)(struct wl_client *client,
			    struct wl_resource *resource,
			    struct wl_resource *surface,
			    struct wl_resource *output);
};

#ifdef  __cplusplus
}
#endif

#endif
